/*******************************************************************************
* File Name: SGPIO_Target.c
* Version 1.40
*
* Description:
*  The SGPIO Target component is an interface component that supports the 
*  SFF-8485 Serial GPIO (SGPIO) industry standard, providing support for 4 
*  SAS/SATA hard drives. SGPIO is a 4-wire bus used between a host bus adaptor
*  (HBA) and a backplane. Of the 4 signals, 3 are driven by the SGPIO Initiator
*  in the HBA (sclock, sload and sdataout) and 1 by the SGPIO Target in the 
*  backplane controller (sdatain). Typically, the HBA is a storage controller
*  located inside a server, desktop, rack or workstation computer which 
*  interfaces with hard disk drives (HDDs) to store and retrieve data
* 
*  Refer to AN64315 "PSoC(R) 3 and PSoC 5 - SFF-8485 Serial GPIO (SGPIO) 
*  Target" for more details and example projects.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/
#include "cytypes.h"
#include "cyfitter.h"
#include "cypins.h"
#include "SGPIO_Target.h"

/*******************************************************************************
* Variables
********************************************************************************/
uint8  SGPIO_Target_initVar = 0u;

/*******************************************************************************
* Function Name: SGPIO_Target_Start()
********************************************************************************
* Summary:
*  Initializes component if not already initialized, then re-enables it
* 
* Parameters:
*  None
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_Start(void)
{ 
    /* If not already initialized, then initialize hardware and software */
    if(SGPIO_Target_initVar == 0u)
    {
        SGPIO_Target_Init();
        SGPIO_Target_initVar = 1u;
    }
    SGPIO_Target_Enable();
}

/*******************************************************************************
* Function Name: SGPIO_Target_Stop()
********************************************************************************
* Summary:
*  Disables the datapath and count7 hardware blocks. 
* 
* Parameters:
*  None
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_Stop(void)
{
    /* Disable the block */
    SGPIO_Target_CONTROL_REG = 0u;
    
    // Disable Count7 hardware block //
    SGPIO_Target_CNT7_AUX_CTL_REG &= ~SGPIO_Target_COUNT7_EN;
}

/*******************************************************************************
* Function Name: SGPIO_Target_Init()
********************************************************************************
* Summary:
*  Enables the datapath and count7 hardware blocks. Sets the SLOAD
*  synchronization match pattern (5 0's followed by a 1).
* 
* Parameters:
*  None
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_Init(void) 
{
    // Reset transmit and receive data buffers in the datapath FIFOs //
    SGPIO_Target_ClearTxBuffer();
    SGPIO_Target_ClearRxBuffer();
    
    // Load SLOAD match pattern ('1' followed by 5 '0's) into datapath D1 //
    SGPIO_Target_DP_D1_REG = SGPIO_Target_SLOAD_SYNC;
}

/*******************************************************************************
* Function Name: SGPIO_Target_Enable()
********************************************************************************
* Summary:
*  Enables the datapath and count7 hardware blocks
* 
* Parameters:
*  None
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_Enable(void) 
{
    // Enable Count7 hardware block //
    SGPIO_Target_CNT7_AUX_CTL_REG |= SGPIO_Target_COUNT7_EN;

    /* Enable the block */
    SGPIO_Target_CONTROL_REG |= SGPIO_Target_EN;
}    

/*******************************************************************************
* Function Name: SGPIO_Target_EnableInt()
********************************************************************************
* Summary:
*  Enables interrupts from the SGPIO_Target
* 
* Parameters:
*  None
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_EnableInt(void) 
{
    SGPIO_Target_CONTROL_REG |= SGPIO_Target_INT_EN;
}

/*******************************************************************************
* Function Name: SGPIO_Target_DisableInt()
********************************************************************************
* Summary:
*  Disables interrupts from the SGPIO_Target
* 
* Parameters:
*  None
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_DisableInt(void) 
{
    SGPIO_Target_CONTROL_REG &= ~SGPIO_Target_INT_EN;
}

/*******************************************************************************
* Function Name: SGPIO_Target_GetIntStatus()
********************************************************************************
* Summary:
*  Returns the interrupt status from the SGPIO_Target
* 
* Parameters:
*  None
*
* Return:
*  0 = no interrupt pending
*  non-zero = interrupt pending
* 
* Side Effects:
*  None
********************************************************************************/
uint8 SGPIO_Target_GetIntStatus(void) 
{
    return (SGPIO_Target_STATUS_REG & SGPIO_Target_RX_BUFF_AVAIL);
}

/*******************************************************************************
* Function Name: SGPIO_Target_GetStatus()
********************************************************************************
* Summary:
*  Returns all status bits from the SGPIO_Target
* 
* Parameters:
*  None
*
* Return:
*  b0 = SGPIO bus synchronization (1=fully sync'ed. >2 SLOAD pattern matches)
*  b1 = bus activity detect sticky bit (1=no bus activity, 0=bus acitivity)
*  b2 = rx buffer available (1=at least one data word received from Initiator)
*  b3 = tx buffer available (1=transmit FIFO has space for at least one data word)
*  b4..7 = not used. Returns all zeroes
* 
* Side Effects:
*  None
********************************************************************************/
uint8 SGPIO_Target_GetStatus(void)
{
    return (SGPIO_Target_STATUS_REG);
}
/*******************************************************************************
* Function Name: SGPIO_Target_WriteTxData(uint16 txData)
********************************************************************************
* Summary:
*  Writes a new frame of data ready to the datapath FIFO ready for transmission
*  to the SGPIO Initiator over SDIN on the next frame.
* 
* Parameters:
*  uint16 txData - 12bit data frame (3 bits per drive, drive 0 is least signficant)
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_WriteTxData(uint16 txData) 
{
    CY_SET_REG16(SGPIO_Target_FIFO_F0_PTR, (txData & 0x0FFFu));
}

/*******************************************************************************
* Function Name: SGPIO_Target_ReadRxData()
********************************************************************************
* Summary:
*  Reads the most recent frame of data received from the SGPIO Initiator.
*  If capturing the vendor specific L3:0 data is enabled, this API needs to be 
*  called twice per interrupt. The first call returns the SDOUT frame. The 2nd
*  call returns the SLOAD L3:0 vendor specific data.
*
*  If capturing the vendor specific data is disabled, this API only needs to be
*  called once per interrupt and will return the SDOUT frame received only.
* 
* Parameters:
*  None
*
* Return:
*  uint16 - 12bit data frame from Initiator
* 
* Side Effects:
*  None
********************************************************************************/
uint16 SGPIO_Target_ReadRxData(void) 
{
    return (CY_GET_REG16(SGPIO_Target_FIFO_F1_PTR) >> 4);
}

/*******************************************************************************
* Function Name: SGPIO_Target_ClearTxBuffer()
********************************************************************************
* Summary:
*  Resets and empties the transmit buffer FIFO
* 
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_ClearTxBuffer(void) 
{
    /* Reset F0 FIFO  */
    CY_SET_REG16(SGPIO_Target_DP_AUX_CTL_PTR, (CY_GET_REG16(SGPIO_Target_DP_AUX_CTL_PTR) |  SGPIO_Target_F0_CLR));
    CY_SET_REG16(SGPIO_Target_DP_AUX_CTL_PTR, (CY_GET_REG16(SGPIO_Target_DP_AUX_CTL_PTR) & ~SGPIO_Target_F0_CLR));

 	/* Initialize F0 to zero in readiness for first sync pattern */
   CY_SET_REG16(SGPIO_Target_FIFO_F0_PTR, 0x0000u);
}

/*******************************************************************************
* Function Name: SGPIO_Target_ClearRxBuffer()
********************************************************************************
* Summary:
*  Resets and empties the receive buffer FIFO
* 
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_ClearRxBuffer(void) 
{
    // Reset F1 and set F1 level to 1 deep or 2 deep depending on vendor specific support //
    #if (SGPIO_Target_VendorSpecific)
        CY_SET_REG16(SGPIO_Target_DP_AUX_CTL_PTR, (CY_GET_REG16(SGPIO_Target_DP_AUX_CTL_PTR) | SGPIO_Target_F1_CLR | SGPIO_Target_F1_LVL));
    #else    
        CY_SET_REG16(SGPIO_Target_DP_AUX_CTL_PTR, (CY_GET_REG16(SGPIO_Target_DP_AUX_CTL_PTR) | SGPIO_Target_F1_CLR));
    #endif
    CY_SET_REG16(SGPIO_Target_DP_AUX_CTL_PTR, (CY_GET_REG16(SGPIO_Target_DP_AUX_CTL_PTR) & ~SGPIO_Target_F1_CLR));
}

/*******************************************************************************
* Function Name: SGPIO_Target_ResetTimeout()
********************************************************************************
* Summary:
*  Resets the logic that checks for no bus activity. This API needs to set bit
*  2 of the control register and then clear it immediately to enable bus activity
*  monitoring. The GetTimeoutStatus() API should be called periodaically to check
*  for bus activity.
* 
* Parameters:
*  None
*
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_ResetTimeout(void) 
{
    SGPIO_Target_CONTROL_REG |=  SGPIO_Target_BUS_ACT_RST;
    SGPIO_Target_CONTROL_REG &= ~SGPIO_Target_BUS_ACT_RST;
}

/*******************************************************************************
* Function Name: SGPIO_Target_GetTimeoutStatus()
********************************************************************************
* Summary:
*  Returns the status of the SGPIO bus activity detection logic
* 
* Parameters:
*  None
*
* Return:
*  0 = bus activity since the last SGPIO_Target_ResetTimeout() API call
*  non-zero = no bus activity since the last SGPIO_Target_ResetTimeout() API call
*  
* 
* Side Effects:
*  None
********************************************************************************/
uint8 SGPIO_Target_GetTimeoutStatus(void) 
{
    return (SGPIO_Target_STATUS_REG & SGPIO_Target_BUS_ACT_BIT);
}

/* [] END OF FILE */
