/*******************************************************************************
 *
 * FILENAME:         SGPIO_Target_v1_40.v
 * Component Name:   SGPIO_Target_v1_40
 *
 * DESCRIPTION:
 *  The SGPIO Target component is an interface component that supports the 
 *  SFF-8485 Serial GPIO (SGPIO) industry standard, providing support for 4 
 *  SAS/SATA hard drives. SGPIO is a 4-wire bus used between a host bus adaptor
 *  (HBA) and a backplane. Of the 4 signals, 3 are driven by the SGPIO Initiator
 *  in the HBA (sclock, sload and sdataout) and 1 by the SGPIO Target in the 
 *  backplane controller (sdatain). Typically, the HBA is a storage controller
 *  located inside a server, desktop, rack or workstation computer which 
 *  interfaces with hard disk drives (HDDs) to store and retrieve data
 *
 *******************************************************************************
 *                 Datapath Register Definitions (16-bit)
 *******************************************************************************
 *
 *  INSTANCE NAME:  TargetShiftReg
 *
 *  DESCRIPTION:
 *    A custom shift register compliant with the SGPIO specification. Transmit
 *    (sdatain) and receive (sdataout) simultaneously, much like standard SPI,
 *    but the data packets are always fixed at 12-bits and are shifted in/out 
 *    continuously.
 *
 *    A synchronization signal (sload) is used to delineate the start and end
 *    of SGPIO frames. The sload signal can also optionally contain additional
 *    "vendor specific" data that gets shifted out of the Initiator 
 *    simultaneously with sdataout.
 *
 *    This component basically implement 2x receive shift registers (sdataout, 
 *    sload) and one transmit shift register (sdatain) in a single datapath.
 *
 *  REGISTER USAGE:
 *    F0 => transmit buffer for sdatain
 *    F1 => receive buffer from sdataout
 *    D0 => not used
 *    D1 => sload sync pulse pattern match
 *    A0 => sdatain/sdataout shift register
 *    A1 => sload/vendor specific shift register
 *
 *  DATA PATH STATES:
 *    0 0 0   0   Pass
 *    0 0 1   1   Pass
 *    0 1 0   2   Shift A1 sload/vendor specific data in
 *    0 1 1   3   Shift A1 sload/vendor specific data in
 *    1 0 0   4   Shift A0 sdataout/sdatain
 *    1 0 1   5   Shift A0 sdataout/sdatain, F0->A0
 *    1 1 0   6   Pass
 *    1 1 1   7   Pass
 *
 ********************************************************************************
 *                 I*O Signals:
 ********************************************************************************
 *    Name              Direction       Description
 *    clock             input           Timing reference (>8x SGPIO sclock frequency)
 *    sclock            input           SGPIO clock in (100 kHz max per specification)
 *    sload             input           SGPIO serial load synchronization signal
 *    sdataout          input           SGPIO serial data out (from Initiator's perspective)
 *    sdatain           output          SGPIO serial data in (from Initiator's perspective)
 *    interrupt         output          Interrupt terminal
 *
 ********************************************************************************
 * Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
 * You may use this file only in accordance with the license, terms, conditions,
 * disclaimers, and limitations in the end user license agreement accompanying
 * the software package with which this file was provided.
 *********************************************************************************/
`include "cypress.v"
`ifdef SGPIO_Target_v1_40_ALREADY_INCLUDED
`else
`define SGPIO_Target_v1_40_ALREADY_INCLUDED
module SGPIO_Target_v1_40 
(
    input   wire    clock,                  /* Timing reference - at least 8x the desired SGPIO sclock frequency */
    input   wire    sclock,                 /* SGPIO clock in (100 kHz max per specification) */
    input   wire    sload,                  /* SGPIO serial load synchronization signal */
    input   wire    sdataout,               /* SGPIO serial data out (from Initiator's perspective) */
    output  reg     sdatain,                /* SGPIO serial data in (from Initiator's perspective) */
    output  wire    interrupt               /* Interrupt terminal */
);

    /**************************************************************************/
    /* Parameters                                                             */
    /**************************************************************************/
    parameter   VendorSpecific = 1;        /* Parameter passed from the customizer */

    /* SGPIO bus link sync detect state machine state definitions */
    localparam  NOT_SYNC  = 2'b00;
    localparam  HALF_SYNC = 2'b10;
    localparam  FULL_SYNC = 2'b11;
    
    /* Device Family and Silicon IDs */
    localparam PSOC3_ES2 = ((`CYDEV_CHIP_MEMBER_USED == `CYDEV_CHIP_MEMBER_3A) && 
                            (`CYDEV_CHIP_REVISION_USED <= `CYDEV_CHIP_REVISION_3A_ES2));                  
    localparam PSOC5_ES1 = ((`CYDEV_CHIP_MEMBER_USED == `CYDEV_CHIP_MEMBER_5A) && 
                            (`CYDEV_CHIP_REVISION_USED <= `CYDEV_CHIP_REVISION_5A_ES1));

    /**************************************************************************/
    /* Internal Signals                                                       */
    /**************************************************************************/
    reg             sclock_d;               /* One clock cycle delayed version of SGPIO sclock */
    reg             sload_d;                /* One clock cycle delayed version of SGPIO sload */
    reg             falling_sclock;         /* SGPIO sclock falling edge detected */
    reg             rising_sclock;          /* SGPIO sclock rising edge detected */
    reg     [1:0]   sync_sm;                /* SGPIO bus link synchronization state machine */
    reg             tc_d;                   /* One clock cycle delayed version of Count7 terminal count output */
    reg             f1_load;                /* Used to control the loading of FIFO F1 from the datapath ALU */
    reg             bus_act_bit;            /* Logic to detect SGPIO bus activity */
    reg             route_si;               /* Shift register input to the datapath */
    wire            synced_clock;           /* Internal clock net synchronized to bus_clk */
    wire    [7:0]   control;                /* SGPIO Target component control register */
    wire    [7:0]   status;                 /* SGPIO Target component status register */
    wire    [6:0]   counter;                /* Count7 counter output */
    wire            tc;                     /* Count7 terminal count output */
    wire            full_sync;              /* Flag indicating full SGPIO bus link synchronization */
    wire    [2:0]   cs_addr;                /* Datapath control store address */
    wire            sr_so;                  /* Shift register output from the datapath */
    wire            rx_buf_avail;           /* Datapath FIFO F1 bus status used to generate interrupts */
    wire            tx_buf_avail;           /* Datapath FIFO F0 bus status */
    wire            sync_dp;                /* SGPIO sload pattern match detected output from the datapath compare logic */
    wire            nc1, nc2, nc3, nc4;     /* nc bits connected to unused datapath output flags */

    /**************************************************************************/
    /* Clock Synchronization                                                  */
    /**************************************************************************/
    cy_psoc3_udb_clock_enable_v1_0 #(.sync_mode(`TRUE)) ClkSync
    (
        /* input  */    .clock_in(clock),
        /* input  */    .enable(1'b1),
        /* output */    .clock_out(synced_clock)
    );  

    /**************************************************************************/
    /* General Purpose Control and Status Registers                           */
    /**************************************************************************/
    /* Target Control Register */
    generate
    if (PSOC3_ES2 || PSOC5_ES1)
    begin: AsyncCtlTCR
        cy_psoc3_control #(.cy_init_value (8'b00000000), .cy_force_order(`TRUE)) TargetControlReg
        (
            /* output [07:00] */  .control(control)
        );
    end
    else
    begin: SyncCtlTCR
        cy_psoc3_control #(.cy_init_value (8'b00000000), .cy_force_order(`TRUE), .cy_ctrl_mode_1(8'h0), .cy_ctrl_mode_0(8'hFF)) TargetControlReg
        (
            /* input          */  .clock(synced_clock),
            /* output [07:00] */  .control(control)
        );
    end
    endgenerate
    wire sgpio_enable = control[0];         /* Component enable */
    wire bus_act_rst  = control[1];         /* 64 ms timeout logic enable */
    wire int_enable   = control[2];         /* Interrupt enable */

    /* Target Status Register */
    cy_psoc3_status #(.cy_force_order(`TRUE), .cy_md_select(8'b00000000)) TargetStatusReg
    (
        /* input         */ .clock(1'b0),
        /* input [07:00] */ .status(status)
    );    
    assign status[0] = full_sync;           /* Status of the SGPIO bus link synchronization (1=synchronized) */
    assign status[1] = bus_act_bit;         /* Status bit to check the 64ms timeout logic (1=no bus activity) */
    assign status[2] = rx_buf_avail;        /* Rx FIFO available (1=data received and available in the F1 FIFO) */
    assign status[3] = tx_buf_avail;        /* Tx FIFO available (1=OK for CPU to write more data into the F0 FIFO) */
    assign status[4] = 1'b0;
    assign status[5] = 1'b0;
    assign status[6] = 1'b0;
    assign status[7] = 1'b0;

    /**************************************************************************/
    /* Count7 block used to count bits on the SGPIO frame                     */
    /**************************************************************************/
    cy_psoc3_count7 #(.cy_period(7'd09),.cy_route_ld(`TRUE),.cy_route_en(`TRUE),.cy_alt_mode(`FALSE)) TargetBitCounter
    (
        /*  input          */  .clock(synced_clock),
        /*  input          */  .reset(1'b0),
        /*  input          */  .load(sync_dp),                  /* Re-load the counter on sload sync pulse */
        /*  input          */  .enable(falling_sclock),         /* Counter enabled on falling edges of SGPIO sclock */
        /*  output [06:00] */  .count(counter),
        /*  output         */  .tc(tc)
     );
    always @(posedge synced_clock) 
    begin
        if (falling_sclock)                                     /* One clock cycle delayed version of tc for the state machine */
            tc_d <= tc;
    end        

    /**************************************************************************/
    /* SGPIO bus clock edge detection and sload sync detect state machine     */
    /**************************************************************************/
    always @(posedge synced_clock) 
    begin
        sclock_d  <= sclock;
        falling_sclock <= ~sclock &  sclock_d;
        rising_sclock  <=  sclock & ~sclock_d;
    end

    /* SGPIO bus link sync detect state machine */
    always @(posedge synced_clock) 
    begin
        if (falling_sclock)
            case (sync_sm) 
            
                /* No sload sync patterns detected */
                NOT_SYNC:
                    if (sync_dp)
                        sync_sm <= HALF_SYNC;
                        
                /* One sload sync pattern detected */
                HALF_SYNC:
                    if (tc_d)
                    begin
                        if (sync_dp)
                            sync_sm <= FULL_SYNC;
                        else
                            sync_sm <= NOT_SYNC;
                    end
                
                /* Two consecutive sload sync patterns detected */
                FULL_SYNC:
                    if (tc_d)
                    begin
                        if (~sync_dp)
                            sync_sm <= NOT_SYNC;
                    end
        
                default: sync_sm <= NOT_SYNC;
            endcase
    end
    assign full_sync = sync_sm[0];
  
    /**************************************************************************/
    /* Shift register timing and control logic                                */
    /**************************************************************************/
    /* Datapath sload/sdata shift register input */ 
    always @(posedge synced_clock)
    begin
        /* sload is valid on falling edges of sclock */
        if (falling_sclock)
            sload_d <= sload;
        if (sclock_d)
            route_si <= sdataout;
        else
            route_si <= sload_d;
    end

    /* Datapath shift register output drives SGPIO sdatain output to Initiator */
    always @(posedge synced_clock)
    begin
        /* sdatain must be driven on rising edges of sclock */
        if (sclock & ~sclock_d & sgpio_enable)
            sdatain <= (sr_so & full_sync);
        /* if SGPIO block is disabled, force the output high */
        else if (~sgpio_enable)
            sdatain <= 1'b1;
    end    
    
    /* Capture shift register data into F1 receive FIFO */
    always @(posedge synced_clock)
    begin
        /* Capture sdataout data coming in from SGPIO Initiator into the F1 FIFO */
        if (~sclock & sclock_d & tc & full_sync)
            f1_load <= 1'b1;
        /* Capture L3:0 vendor specific data coming in from SGPIO Initiator into the F1 FIFO */
        else if(sclock & ~sclock_d & tc_d & full_sync & VendorSpecific)
            f1_load <= 1'b1;
        else
            f1_load <= 1'b0;
    end
    
    /**************************************************************************/
    /* Interrupt generation logic                                             */
    /**************************************************************************/
    /* Since Tx and Rx occur simultaneously and continuously, generate an interrupt when we receive new data */
    assign interrupt = sgpio_enable & int_enable & rx_buf_avail;

    /**************************************************************************/
    /* Support for SGPIO bus 64 ms no activity timeout                        */
    /*                                                                        */
    /* The firmware API ResetTimeout() periodically arms the bus activity bit */
    /* (bus_act_bit) by setting it high through the control register          */
    /* (bus_act_rst). Once the bit is armed, it will go low if there is any   */
    /* low transition on any of the 3 SGPIO Initiator signals (sclock, sload, */
    /* sdataout). If there is no activity on the bus, then the armed bit will */
    /* remain high. The firmware API GetTimeoutStatus() can be used to monitor*/
    /* bus_act_bit. According to the SFF-8485 spec, no acitivy for >64 ms is  */
    /* a timeout condition which must be recognized and appropriate action    */
    /* taken.                                                                 */
    /*                                                                        */
    /* The bus activity bit in hardware along with the provided firmware APIs */
    /* simply give designers a mechanism to detect a bus timeout condition.   */
    /* Application level firmware is required to determine specifically what  */
    /* action to take in that case.                                           */
    /**************************************************************************/
    always @ (posedge synced_clock or posedge bus_act_rst)
    begin
        if (bus_act_rst)
            bus_act_bit <= 1'b1;
        else
            bus_act_bit <= bus_act_bit & sload & sclock & sdataout;
    end

    /**************************************************************************/
    /* 16-bit datapath configured as SGPIO shift registers and parallel to    */
    /* serial interface.                                                      */
    /*                                                                        */
    /* Datapath implements 2 shift registers: A0=sdataout, A1=sload           */
    /* Datapath shifts SGPIO sdataout through A0 on falling edges of sclock   */
    /* Datapath shifts SGPIO sload through A1 on rising edges of sclock       */
    /* Count7 tc bit is used to indicate appropriate time to load new CPU     */
    /* data from FIFO F0 into the shift register                              */
    /**************************************************************************/
    assign cs_addr = {falling_sclock, rising_sclock , tc & sync_sm[1]};
    cy_psoc3_dp16 #(.cy_dpconfig_a(
    {
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_A0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM0: PASS*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM1: PASS*/
        `CS_ALU_OP_PASS, `CS_SRCA_A1, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC_NONE, `CS_A1_SRC__ALU,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM2: Shift A1 (sload)*/
        `CS_ALU_OP_PASS, `CS_SRCA_A1, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC_NONE, `CS_A1_SRC__ALU,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM3: Shift A1 (sload)*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC__ALU, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM4: Shift A0 (sdataout)*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC___F0, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM5: Shift A0 (sdataout), F0->A0*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM6: PASS*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM7: PASS*/
          8'hFF, 8'h00,    /*CFG9:*/
          8'hFF, 8'hFF,    /*CFG11-10:*/
        `SC_CMPB_A1_D1, `SC_CMPA_A0_D1, `SC_CI_B_ARITH,
        `SC_CI_A_ARITH, `SC_C1_MASK_ENBL, `SC_C0_MASK_DSBL,
        `SC_A_MASK_DSBL, `SC_DEF_SI_0, `SC_SI_B_DEFSI,
        `SC_SI_A_CHAIN, /*CFG13-12:*/
        `SC_A0_SRC_ACC, `SC_SHIFT_SR, 1'h0,
        1'h0, `SC_FIFO1_ALU, `SC_FIFO0_BUS,
        `SC_MSB_DSBL, `SC_MSB_BIT7, `SC_MSB_NOCHN,
        `SC_FB_NOCHN, `SC_CMP1_NOCHN,
        `SC_CMP0_NOCHN, /*CFG15-14:*/
         10'h00, `SC_FIFO_CLK__DP,`SC_FIFO_CAP_AX,
        `SC_FIFO_LEVEL,`SC_FIFO__SYNC,`SC_EXTCRC_DSBL,
        `SC_WRK16CAT_DSBL /*CFG17-16:*/
    }), .cy_dpconfig_b(
    {
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_A0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM0: PASS*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM1: PASS*/
        `CS_ALU_OP_PASS, `CS_SRCA_A1, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC_NONE, `CS_A1_SRC__ALU,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM2: Shift A1 (sload)*/
        `CS_ALU_OP_PASS, `CS_SRCA_A1, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC_NONE, `CS_A1_SRC__ALU,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM3: Shift A1 (sload)*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC__ALU, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM4: Shift A0 (sdataout)*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP___SR, `CS_A0_SRC___F0, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM5: Shift A0 (sdataout), F0->A0*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM6: PASS*/
        `CS_ALU_OP_PASS, `CS_SRCA_A0, `CS_SRCB_D0,
        `CS_SHFT_OP_PASS, `CS_A0_SRC_NONE, `CS_A1_SRC_NONE,
        `CS_FEEDBACK_DSBL, `CS_CI_SEL_CFGA, `CS_SI_SEL_CFGA,
        `CS_CMP_SEL_CFGB, /*CFGRAM7: PASS*/
          8'hFF, 8'h00,    /*CFG9:*/
          8'hFC, 8'hFF,    /*CFG11-10: sload Sync Detection*/
        `SC_CMPB_A1_D1, `SC_CMPA_A1_D1, `SC_CI_B_ARITH,
        `SC_CI_A_ARITH, `SC_C1_MASK_ENBL, `SC_C0_MASK_DSBL,
        `SC_A_MASK_DSBL, `SC_DEF_SI_0, `SC_SI_B_DEFSI,
        `SC_SI_A_ROUTE, /*CFG13-12:*/
        `SC_A0_SRC_ACC, `SC_SHIFT_SR, 1'h0,
        1'h0, `SC_FIFO1_ALU, `SC_FIFO0_BUS,
        `SC_MSB_DSBL, `SC_MSB_BIT7, `SC_MSB_NOCHN,
        `SC_FB_NOCHN, `SC_CMP1_NOCHN,
        `SC_CMP0_NOCHN, /*CFG15-14:*/
         10'h00, `SC_FIFO_CLK__DP,`SC_FIFO_CAP_AX,
        `SC_FIFO_LEVEL,`SC_FIFO__SYNC,`SC_EXTCRC_DSBL,
        `SC_WRK16CAT_DSBL /*CFG17-16:*/
    })) TargetShiftReg 
    (
        /*  input                   */  .clk(synced_clock),
        /*  input   [02:00]         */  .cs_addr(cs_addr),
        /*  input                   */  .route_si(route_si),                /* SGPIO sdataout or sload */
        /*  input                   */  .route_ci(1'b0),
        /*  input                   */  .f0_load(1'b0),
        /*  input                   */  .f1_load(f1_load),        
        /*  input                   */  .d0_load(1'b0),
        /*  input                   */  .d1_load(1'b0),
        /*  output  [01:00]         */  .ce0(),
        /*  output  [01:00]         */  .cl0(),
        /*  output  [01:00]         */  .z0(),
        /*  output  [01:00]         */  .ff0(),
        /*  output  [01:00]         */  .ce1({sync_dp, nc1}),               /* SGPIO sload sync pattern detect */
        /*  output  [01:00]         */  .cl1(),
        /*  output  [01:00]         */  .z1(),
        /*  output  [01:00]         */  .ff1(),
        /*  output  [01:00]         */  .ov_msb(),
        /*  output  [01:00]         */  .co_msb(),
        /*  output  [01:00]         */  .cmsb(),
        /*  output  [01:00]         */  .so({nc2, sr_so}),                  /* SGPIO sdatain */
        /*  output  [01:00]         */  .f0_bus_stat({tx_buf_avail, nc3}),
        /*  output  [01:00]         */  .f0_blk_stat(),
        /*  output  [01:00]         */  .f1_bus_stat({rx_buf_avail, nc4}),
        /*  output  [01:00]         */  .f1_blk_stat()
    );
endmodule
`endif /* SGPIO_Target_v1_40_ALREADY_INCLUDED */
