/*******************************************************************************
* File Name: SGPIO_Target_PM.c
* Version 1.40
*
* Description:
*  This file contains the power management APIs for the SGPIO Target
*  component.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/
#include "SGPIO_Target.h"

/* Backup structure just stores component's enable state prior to sleep */
static SGPIO_Target_BACKUP_STRUCT SGPIO_Target_backup = {0u, 0u};

/*******************************************************************************
* Function Name: SGPIO_Target_SaveConfig
********************************************************************************
* Summary:
*  This API is used to save the configuration of the SGPIO_Target component prior
*  to PSoC entering low-power sleep mode. No register/parameters are required
*  to be saved.
* 
* Parameters:
*  None
* 
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_SaveConfig(void)
{
    /* Save component's interrupt enable state */
    if ((SGPIO_Target_CONTROL_REG & SGPIO_Target_INT_EN) == SGPIO_Target_INT_EN)
    {
        SGPIO_Target_backup.interruptEnableState = 1u;
    }
    /* Interrupts are disabled */
    else
    {
        SGPIO_Target_backup.interruptEnableState = 0u;
    }
}

/*******************************************************************************
* Function Name: SGPIO_Target_RestoreConfig
********************************************************************************
* Summary:
*  This API is used to restore the configuration of the SGPIO_Target component
*  after PSoC exits low-power sleep mode. No register/parameters are required
*  to be restored.
*
* Parameters:
*  None
* 
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_RestoreConfig(void) 
{
    /* Restore component's interrupt enable state */
    if (SGPIO_Target_backup.interruptEnableState == 1u)
    {
        SGPIO_Target_CONTROL_REG |= SGPIO_Target_INT_EN;
    }
 }

/*******************************************************************************
* Function Name: SGPIO_Target_Sleep
********************************************************************************
* Summary:
*  This API is used to configure the SGPIO_Target component for minimum power 
*  consumption prior to PSoC entering low-power sleep mode. Save the enable
*  state of the component, stop it and backup non-retention data to SRAM.
* 
* Parameters:
*  None
* 
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_Sleep(void)
{
    /* Save component's enable state */
    if ((SGPIO_Target_CONTROL_REG & SGPIO_Target_EN) == SGPIO_Target_EN)
    {
        SGPIO_Target_backup.enableState = 1u;
    }
    /* Component is disabled */
    else
    {
        SGPIO_Target_backup.enableState = 0u;
    }
    SGPIO_Target_SaveConfig();
    SGPIO_Target_Stop();
}

/*******************************************************************************
* Function Name: SGPIO_Target_WakeUp
********************************************************************************
* Summary:
*  This API is used to configure the SGPIO_Target component for active power 
*  consumption after PSoC exits low-power sleep mode. Restore non-retention data
*  from SRAM. Re-initialize the component and re-enable it if it was enabled
*  before going to sleep
*
* Parameters:
*  None
* 
* Return:
*  None
* 
* Side Effects:
*  None
********************************************************************************/
void SGPIO_Target_WakeUp(void) 
{
    SGPIO_Target_RestoreConfig();

    /* Re-initialize the component */
    SGPIO_Target_Init();
    
    /* Restore component's enable state prior to going to sleep */
    /* Do nothing if component was disabled */
    if (SGPIO_Target_backup.enableState != 0u)
    {
         /* Components block was enabled */
         SGPIO_Target_Enable();
    }
}

/* [] END OF FILE */
